package piy;

import piy.action.IfAction;
import piy.support.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.lang.reflect.*;
import java.beans.PropertyChangeListener;

import java.util.*;
import java.io.*;

/**
* The window in which action lists can be edited/created.
* @author David Vivash
* @version 1.0.4, 28/04/01
*/
public class ActionListEditor extends JFrame implements ChangeListener, ActionListener, Observer 
{
	private static String NEW_ACTIONLIST	= "newActionList";
	private static String DELETE_ACTIONLIST = "deleteActionList";
	private static String INSERT_BEFORE		= "insertBefore";
	private static String INSERT_AFTER		= "insertAfter";
	private static String DELETE_ACTION		= "deleteAction";

	private ActionList list = null; //the action list being editted

	private ActionDescriptor[] actionDescriptors;

	private ScrollingButtonPanel[] actionPanel 		= null;
	private JTabbedPane tabPane 					= null;
	private ActionListNamePanel actionListNamePanel	= null;
	private PropertiesPanel propertiesPanel 		= new PropertiesPanel();
	private ActionListViewer viewer 				= null;
	private JScrollPane scroller 					= null;

	//The action descriptor that has currently been selected on the toolbar
	private ActionDescriptor toolbarSelection = null;

	/**
	* Construct a new ActionListEditor window viewing the specified action list.  If the specified
	* list has not yet been added to the project currently being editted, this constructor
	* automatically adds it.
	* @param actionDescriptors the descriptor classes for all of the available actions
	* @param list the list to be editted in the window
	*/
	public ActionListEditor(ActionDescriptor[] actionDescriptors, ActionList list) {
		super("Action List Editor");

		PIYNamer actionListNamer = ProjectHandler.getInstance().getProject().getActionListNamer();

		//if the list doesn't have a name...
//		if ((list != null) && (actionListNamer.getName(list) == null))
//			actionListNamer.add(list, "ActionList");

		this.list = list;
		this.actionDescriptors = actionDescriptors;
		
		PIY.getInstance().addObserver(this);
		
		startup();
	}

	/**
	* Called by the constructor to set up the initial frame structure.
	*/
	private void startup() {
		getContentPane().setLayout(new BorderLayout());

		JPanel topPanel = new JPanel();
		topPanel.setLayout(new BorderLayout());

		//---- Set up toolbar buttons -------
		JPanel toolbarButtons = new JPanel();

		toolbarButtons.setLayout(null);
		toolbarButtons.setPreferredSize(new Dimension(64,60));
		
		String sep = File.separator;
		String images = "piy" + sep + "images" + sep;

		FloatingButton newActionList = new FloatingButton(new ImageIcon(images + "new.gif").getImage(), NEW_ACTIONLIST, false);
		newActionList.addActionListener(this);
		newActionList.setBounds(4, 6, 22, 22);
		newActionList.setToolTipText("New Action List");

		FloatingButton removeAction = new FloatingButton(new ImageIcon(images + "bin.gif").getImage(), DELETE_ACTION, false);
		removeAction.addActionListener(this);
		removeAction.setBounds(30, 6, 22, 22);
		removeAction.setToolTipText("Remove Action");

		FloatingButton insertBefore = new FloatingButton(new ImageIcon(images + "insertbefore.gif").getImage(), INSERT_BEFORE, false);
		insertBefore.addActionListener(this);
		insertBefore.setBounds(4, 34, 22, 22);
		insertBefore.setToolTipText("Insert Before");

		FloatingButton insertAfter = new FloatingButton(new ImageIcon(images + "insertafter.gif").getImage(), INSERT_AFTER, false);
		insertAfter.addActionListener(this);
		insertAfter.setBounds(30, 34, 22, 22);
		insertAfter.setToolTipText("Insert After");

		toolbarButtons.add(newActionList);
		toolbarButtons.add(removeAction);
		toolbarButtons.add(insertBefore);
		toolbarButtons.add(insertAfter);

		//------ Set up action buttons panel -----
		tabPane = new JTabbedPane();
		ArrayList collections = sortCategories();
		
		actionPanel = new ScrollingButtonPanel[collections.size()];

		FloatingButton toolButton		= null;
		ActionDescriptor[] descriptors	= null;
		
		for (int i=0; i<collections.size(); i++) {
			descriptors = (ActionDescriptor[])collections.get(i);
			actionPanel[i] = new ScrollingButtonPanel(descriptors.length, 26, 26, 4);
			tabPane.addTab(descriptors[0].getCategory(), actionPanel[i]);
			actionPanel[i].addChangeListener(this);

			for (int j=0; j<descriptors.length; j++) {
				actionPanel[i].addButton(toolButton = new FloatingButton(descriptors[j].getIcon(), ""+j, true));

				//where the action descriptor is located in the actionDescriptors array
				int num = findDescriptor(descriptors[j]);
				toolButton.setIdentifier(num);

				toolButton.setToolTipText(descriptors[j].getName());
			}
		}
		
		//Move the "General" tab to be the first tab in the pane
		int posOfGeneral = tabPane.indexOfTab("General");
		if (posOfGeneral != -1) {
			if (posOfGeneral != 0) { 
				Component generalPanel = tabPane.getComponentAt(posOfGeneral);
				tabPane.removeTabAt(posOfGeneral);
				tabPane.insertTab("General", null, generalPanel, null, 0);
			}
		}
		
		
		topPanel.add(toolbarButtons, BorderLayout.WEST);
		topPanel.add(tabPane, BorderLayout.CENTER);

		JPanel padding = new JPanel();
		padding.setPreferredSize(new Dimension(0,8));
		topPanel.add(padding, BorderLayout.SOUTH);

		JPanel leftPanel = new JPanel();
		leftPanel.setPreferredSize(new Dimension(180,100));
		leftPanel.setMaximumSize(new Dimension(400,100));
		leftPanel.setMinimumSize(new Dimension(50,100));

		actionListNamePanel = new ActionListNamePanel(list);
		leftPanel.setLayout(new BorderLayout());
		leftPanel.add(actionListNamePanel, BorderLayout.NORTH);
		leftPanel.add(propertiesPanel, BorderLayout.CENTER);


		viewer = new ActionListViewer(list);
		scroller = new JScrollPane(viewer);


		JSplitPane bottomPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, leftPanel, scroller);

		leftPanel.setPreferredSize(new Dimension(180,100));

		getContentPane().add(topPanel, BorderLayout.NORTH);
		getContentPane().add(bottomPanel, BorderLayout.CENTER);
	}
	
	/**
	* Takes the action descriptors, and sorts them into a collection of arrays - each array contains
	* descriptors of actions in the same category.  So sorts out all of the string actions from the
	* files actions, for example.  Each entry in the array list returned is an ActionDescriptor[].
	* @return a collection of ActionDescriptor arrays, with each array containing descriptors in the
	* same category.
	*/
	private ArrayList sortCategories() {
		//generate a map from each category name to the number of actions in that category
		//ie categoryName(String) -> Count(Inteeger)
		HashMap nameToCount = new HashMap(actionDescriptors.length);
		Integer num = null;
	
		for (int i=0; i< actionDescriptors.length; i++) {
			num = (Integer)nameToCount.get(actionDescriptors[i].getCategory());
		
			if (num == null) //category name doesn't yet exist
				num = new Integer(1);
			else 
				num = new Integer(num.intValue() + 1);
			
			nameToCount.put(actionDescriptors[i].getCategory(), num);
		}
	
		//Now construct each array and store it in the arraylist to return.
		//We could return an array of ActionDescriptor arrays, but I think that's a bit
		//too confusing
		ArrayList toReturn = new ArrayList(nameToCount.size());
		ActionDescriptor[] array = null;
		String currentName = null;
		int arrayCounter = 0;

		//Iterate over each named category
		Iterator nameIterator = nameToCount.keySet().iterator();
		while (nameIterator.hasNext()) {
			currentName = (String)nameIterator.next();
			
			array = new ActionDescriptor[((Integer)nameToCount.get(currentName)).intValue()];
			arrayCounter = 0;
			for (int j=0; j<actionDescriptors.length; j++) {
				if (actionDescriptors[j].getCategory().equals(currentName)) {
					array[arrayCounter] = actionDescriptors[j];
					arrayCounter++;
				}
			}
			
			toReturn.add(array);
		}
		
		return toReturn;
	}
	
	/**
	* Find the specified Descriptor's location in the actionDescriptors array
	* @param descriptor the descriptor to search for
	* @return the descriptor's location in the actionDescriptors array, or -1 if it was not found
	*/
	private int findDescriptor(ActionDescriptor descriptor) {
		for (int i=0; i<actionDescriptors.length; i++) 
			if (descriptor == actionDescriptors[i])
				return i;

		return -1;
	}			

	//----- ChangeListener method ------------
	private boolean editting = false;
	public void stateChanged(ChangeEvent e) {
		if (!editting) {
			ScrollingButtonPanel inside = (ScrollingButtonPanel)tabPane.getSelectedComponent();
			int pos = inside.getSelectedIndex();
		
			editting = true;
			//Clear selections on other tabs now
			for (int i=0; i<actionPanel.length; i++) 
				if (actionPanel[i] != inside) actionPanel[i].clearSelection();
			
			//TO DO!!!! Get the action descriptor that was selected
			if (pos == -1) {
				toolbarSelection = null;
			} else {
				FloatingButton button = (FloatingButton)inside.getSelectedButton();
				toolbarSelection = actionDescriptors[button.getIdentifier()];
			}

			
			editting = false;
			
			if (pos == -1) viewer.setEditType(ActionListViewer.SELECT);
			else viewer.setEditType(ActionListViewer.ADD);
		}
	}

	//------ ActionListener method -----------
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();

		if (command.equals(INSERT_BEFORE)) {
			viewer.insertBefore();
			scroller.validate();	
		} else if (command.equals(INSERT_AFTER)) {
			viewer.insertAfter();
			scroller.validate();
		} else if (command.equals(NEW_ACTIONLIST)) {
			viewer.setList(actionListNamePanel.newList());
		} else if (command.equals(DELETE_ACTION)) {
			viewer.removeSelectedAction();
		}

		ProjectHandler.getInstance().projectChanged();
	}

	//--------- Observer method ----------------
	public void update(Observable source, Object arg) {
		ActionEvent e = (ActionEvent)arg;
		String command = e.getActionCommand();
		
		if ((command == PIY.NEW_PROJECT) || (command == PIY.OPEN_PROJECT)) {
			viewer.setList(null);			
			actionListNamePanel.setup(list);
		}
	}
	
	
	/**
	* The actual view of an action list, as a series of boxes (icons).  Each box can be 
	* clicked on - the boxes with icons in them can be dragged.
	* @author David Vivash
	* @version 1.0, 26/02/01
	*/
	private class ActionListViewer extends JPanel implements ActionListener {

		private int arrowWidth		= 16; //space between two actions
		private int ifBeforeWidth	= 32; //space between an "if" and the first action in its lists 
		private int ifAfterWidth	= 24; //space between the last action in an "if"-list and the action after the "if"
		private int spaceHeight		= 4; //extra space to pad between parallel lists

		private ActionIcon selected	= null;			//the currently selected action icon
		private Color selectColor = Color.red;		//the border color for the selected icon
		private int selectThickness = 2;			//the thickness of the border to draw

		public static final int SELECT	= 0;
		public static final int ADD		= 1;

		private int editType = SELECT;

		public ActionListViewer(ActionList list) {
			super();
	
			setBackground(Color.white);

			setLayout(null);

			if (list != null) {
				restructureList(list);
				
				//sets up the list and selects the first action in it
				selectAction(setupList(list, new Point(0,0), new Dimension(getRequiredWidth(list), getRequiredHeight(list)), list.get(0)));
			}
		}
	
		/**
		* Inserts a Null PIYAction before the currently selected icon.
		*/
		public void insertBefore() {
			if (selected != null) {
				selected.getActionList().add(selected.getActionList().getIndex(selected.getPIYAction()), new NullAction());
				removeAll();
				selected = setupList(list, new Point(0,0), new Dimension(getRequiredWidth(list), getRequiredHeight(list)), selected.getPIYAction());
				repaint();
			}
		}
	
		/**
		* Inserts a Null PIYAction after the currently selected icon.
		*/
		public void insertAfter() {
			if (selected != null) {
				selected.getActionList().add(selected.getActionList().getIndex(selected.getPIYAction())+1, new NullAction());
				removeAll();
				selected = setupList(list, new Point(0,0), new Dimension(getRequiredWidth(list), getRequiredHeight(list)), selected.getPIYAction());
				repaint();
			}
		}
		
		/**
		* Removes the currently selected action (if there is one), and then selects null.
		*/
		public void removeSelectedAction() {
			if (selected != null) {
				selected.getActionList().removeAction(selected.getPIYAction());
				restructureList(list);
				removeAll();
				setupList(list, new Point(0,0), new Dimension(getRequiredWidth(list), getRequiredHeight(list)), null);
				selectAction(null);
				repaint();
			}
		
		}

		public void paint(Graphics g) {
			//paint the components
			super.paint(g);
			
			if (list != null) {
				//draw all of the lines			
				paintListLines(list, new Point(0,0), new Dimension(getRequiredWidth(list), getRequiredHeight(list)), g);
	
				//show which actionicon is selected
				if (selected != null) {
					g.setColor(selectColor);
					for (int i=1; i<=selectThickness; i++)
						g.drawRect(selected.getX()-i, selected.getY()-i, selected.getWidth()+2*i-1, selected.getHeight()+2*i-1);
				}
			}
		}

		/**
		* Paints the lines connecting actions in an action list.
		* @param list the list for which the lines should be painted
		* @param position the top left corner coords of the space gien to draw the lines
		* @param size the width and height given to draw this list
		* @param g the graphics context on which the paint the list lines
		*/
		private void paintListLines(ActionList list, Point position, Dimension size, Graphics g) {
			int current = (size.width - getRequiredWidth(list))/2 + (int)position.getX();
			int y = size.height/2 + (int)position.getY();

			PIYAction act = null;

			for (int i=0; i<list.size()-1; i++) {
				act = list.get(i);

				current+=40;

				if (act instanceof IfAction) {
					IfAction ifAct = (IfAction)act;
					int width1 = getRequiredWidth(ifAct.getTrueList());
					int width2 = getRequiredWidth(ifAct.getFalseList());
					
					int reqWidth = width1 > width2 ? width1 : width2;

					paintListLines(ifAct.getTrueList(), new Point(current + ifBeforeWidth, (int)position.getY()), 
									new Dimension(reqWidth, size.height/2), g);
					paintListLines(ifAct.getFalseList(), new Point(current + ifBeforeWidth, (int)position.getY()+size.height/2), 
									new Dimension(reqWidth, size.height/2), g);


					drawIfBranch(current, y, width1, width2, size.height/2, g);

					current += ifBeforeWidth + reqWidth + ifAfterWidth;

				} else {
					drawArrow(current, y, arrowWidth, g);
					current += arrowWidth;
				}

			}

		}

		/**
		* Draws a line with an arrow on the end, from left to right.
		* @param x the leftmost x position of the arrow line
		* @param y the y coordinate of the arrow line
		* @param size the length of the line to draw to the right (length includes arrowhead size)
		* @param g the graphics context on which to draw the arrow
		*/
		private void drawArrow(int x, int y, int size, Graphics g) {
			g.drawLine(x, y, x+size, y);
			g.fillPolygon(new int[] {x+size-5, x+size, x+size-5},
							new int[] {y-3, y, y+3},
									3);
		}

		/**
		* Draws the branch from an IfAction into its two branches, and from the two branches back
		* to the original at the end.
		* @param x the x position (immediately after the IfAction icon)
		* @param y the y position of the branch (midway down the IfAction icon)
		* @param width1 the required width of the first branch
		* @param width2 the required width of the second branch
		* @param height the required height of the tallest branch
		* @param g the graphics context on which the branches should be drawn
		*/
		private void drawIfBranch(int x, int y, int width1, int width2, int height, Graphics g) {
			int reqWidth = width1 > width2 ? width1 : width2;

			g.drawLine(x, y, x+ifBeforeWidth/2, y);
			g.drawLine(x+ifBeforeWidth/2, y-height/2, x+ifBeforeWidth/2, y+height/2);

			drawArrow(x+ifBeforeWidth/2, y-height/2, ifBeforeWidth/2 + (reqWidth - width1)/2, g);
			drawArrow(x+ifBeforeWidth/2, y+height/2, ifBeforeWidth/2 + (reqWidth - width2)/2, g);

			int posAcross = x+reqWidth+ifBeforeWidth+ifAfterWidth/2;

			g.drawLine(x+ifBeforeWidth+width1+(reqWidth - width1)/2, y-height/2, posAcross, y-height/2);
			g.drawLine(x+ifBeforeWidth+width2+(reqWidth - width2)/2, y+height/2, posAcross, y+height/2);

			g.drawLine(posAcross, y-height/2, posAcross, y+height/2);
			drawArrow(posAcross, y, ifAfterWidth/2, g);
		}


		/**
		* Sets up the list view by creating an icon for every action and positioning
		* them accordingly.  (Also restructures the list if needed).
		* @param list the list for which to create and position icons
		* @param position to top left coordinate of the space into which the icons 
		* should be positioned
		* @param size the width and height of the space the collection of icons must 
		* take up
		* @param action the action which is currently selected - you can provide this information
		* so that this method can tell you the reference of the icon created for the action
		* @return the action icon created for the action specified
		*/
		public ActionIcon setupList(ActionList list, Point position, Dimension size, PIYAction action) {

			ActionDescriptor desc	= null;
			ActionIcon current		= null;
			
			ActionIcon icon = null; //this is the icon which will be returned - it should represent the action

			int top = size.height/2-20 + (int)position.getY(); //icon positioned in the middle of the vertical coords

			int currentPos = size.width/2 - getRequiredWidth(list)/2 + (int)position.getX() ;

			for (int i=0; i<list.size(); i++) {

				desc = getDescriptor(list.get(i).getClass());
				add(current = new ActionIcon(list.get(i), list, desc.getBigIcon()));

				if (action == list.get(i)) icon = current;

				current.setBounds(currentPos, top, 40, 40);
				
				current.addActionListener(this);

				currentPos += 40;
			
				if (list.get(i) instanceof IfAction) {

					ActionList trueList, falseList;
			
					int width1 = getRequiredWidth( trueList = ((IfAction)list.get(i)).getTrueList() );
					int width2 = getRequiredWidth( falseList = ((IfAction)list.get(i)).getFalseList());

					int required = (width1 > width2) ? width1 : width2;

					ActionIcon icon1 = setupList(trueList, new Point(currentPos+ifBeforeWidth, (int)position.getY()), new Dimension(required, size.height/2), action);
					ActionIcon icon2 = setupList(falseList, new Point(currentPos+ifBeforeWidth, (int)position.getY()+size.height/2), new Dimension(required, size.height/2), action);

					icon = (icon1 != null) ? icon1 : ((icon2 != null) ? icon2 : icon);

					currentPos += ifBeforeWidth + required + ifAfterWidth;

				} else currentPos += arrowWidth;

			}
			
			selectAction(icon);

			return icon;
		}

		/**
		* Sets the list displayed in the panel.
		* @param actionList the list to display
		*/
		public void setList(ActionList actionList) {
			list = actionList;
			if (list != null) restructureList(list);

			removeAll();
			selected = null;

			//sets up the list and selects the first action in it
			if (list != null)
				selectAction(setupList(list, new Point(0,0), new Dimension(getRequiredWidth(list), getRequiredHeight(list)), list.get(0)));
			else selectAction(null);

			scroller.validate();
			repaint();
		}
		
		/**
		* Retrieve the ActionDescriptor for a particular action class type.
		* @param action the class of the action whose actiondescriptor is required
		* @return the action descriptor of the specified action class, or null if an
		* an appropriate descriptor was not found
		*/
		protected ActionDescriptor getDescriptor(Class action) {
			//The NullAction will not be present in the descriptor array
				if (action == NullAction.class) return NullDescriptor.getInstance();

			for (int i=0; i<actionDescriptors.length; i++)
				if (actionDescriptors[i].getDescribedClass() == action)
					return actionDescriptors[i];
		
			return null;
		}

		/**
		* Deletes the currently selected list and informs the properties panel.  Doesn't inform the
		* action list names combo box.
		*/
		public void deleteList() {
			if (list != null) {
				PIYNamer names = ProjectHandler.getInstance().getProject().getActionListNamer();
				names.removeValue(names.getName(list));	
			}
		}

		/**
		* Goes through a list and restructures it so that the actionlist viewer can view it. The
		* restructuring is to ensure that all IfActions have an action after them, and all
		* actionlists have at least one action.
		* @param list the list to restructure
		*/
		private void restructureList(ActionList list) {
			if (list != null) {
				if (list.size() == 0) list.add(new NullAction());
				//we don't allow the list to end on an IfAction for drawing (aesthetic) reasons
				if (list.get(list.size()-1) instanceof IfAction) list.add(new NullAction());
	
				for (int i=0; i<list.size(); i++) {
					if (list.get(i) instanceof IfAction) {
						restructureList(((IfAction)list.get(i)).getTrueList());
						restructureList(((IfAction)list.get(i)).getFalseList());
					}
				}
			}
		}

		/**
		* Selects the specified action icon, informing the properties panel to display its properties
		* @param action the action icon to select
		*/
		public void selectAction(ActionIcon action) {
			selected = action;
			
			propertiesPanel.setup(selected == null ? null : selected.getPIYAction());
			repaint();
			
		}

		/**
		* Sets the editype behaviour of this viewer when a null action is selected.  If type = SELECT,
		* when the Null PIYAction is clicked it becomes selected.  If type = ADD, when a Null PIYAction is 
		* clicked, it is replaced with the currently selected action type on the toolbar.
		* @param type one of SELECT or ADD
		*/
		public void setEditType(int type) {
			if ((type == SELECT) || (type == ADD))
				editType = type;
		}

		/**
		* This calculates how much horizontal space the actionlist view takes up.  Each
		* action takes up 40 (for the icon) + arrowWidth (for the arrow), unless it is an IfAction,
		* which takes up 40 (for the icon) + space before and after (for the lines) + the amount of space 
		* required by the longest of its two internal actionlists.
		* @param a the action list whose width is to be determined
		* @return the number of pixels required
		*/
		public int getRequiredWidth(ActionList a) {
			if (a == null) return 0;
			
			int width = 0;

			for (int i=0; i<a.size(); i++) {
				width += a.get(i) instanceof IfAction ? 
					40 + ifBeforeWidth + ifAfterWidth + Math.max(getRequiredWidth(((IfAction)a.get(i)).getTrueList()),
										getRequiredWidth(((IfAction)a.get(i)).getFalseList())) :
					40 + arrowWidth;
			}

			return width - arrowWidth;
		}

		/**
		* Calculate how much horizontal space is required by an actionlist.  This 
		* will be 40 if the list is just a sequence of actions, but if the list 
		* includes an IfAction then the list takes 40 + the height of the truelist +
		* the height of the falselist. Note that we take the maximum required height 
		* out of <i>all</i> of the IfActions present in any one actionlist.
		* 
		* @param a the list for which the height should be determined
		* @return the amount of horizontal space the list <i>a</i> requires
		*/
		public int getRequiredHeight(ActionList a) {
			if (a == null) return 0; 
		
			int height = 40;

			for (int i=0; i<a.size(); i++) {
				if (a.get(i) instanceof IfAction) {
					height = Math.max(height,spaceHeight + 2*Math.max( getRequiredHeight(((IfAction)a.get(i)).getTrueList()),
																		getRequiredHeight(((IfAction)a.get(i)).getFalseList())));
				}
			}
			return height;
		}

		public Dimension getMaximumSize() {
			return new Dimension(getRequiredWidth(list),getRequiredHeight(list));
		}
	
		public Dimension getMinimumSize() {
			return new Dimension(getRequiredWidth(list),getRequiredHeight(list));
		}
	
		public Dimension getPreferredSize() {
			return new Dimension(getRequiredWidth(list),getRequiredHeight(list));

		}

		//------------ ActionListener Method -------------
		public void actionPerformed(ActionEvent e) {
			switch (editType) {
				case SELECT:
					selectAction((ActionIcon)e.getSource());
					break;
				case ADD: //Add an action from the toolbar to the list
					selected = (ActionIcon)e.getSource();
					if (selected.getPIYAction() instanceof NullAction) {
						
						int pos = selected.getActionList().removeAction(selected.getPIYAction());
						try{					
							//Add the selected action to the action list
							PIYAction toSelect = (PIYAction)toolbarSelection.getDescribedClass().newInstance();
							selected.getActionList().add(pos, toSelect);

							for (int i=0; i<actionPanel.length; i++)
								actionPanel[i].clearSelection(); //deselect the icon on the toolbar

							removeAll();//clear all of the old action icons
							restructureList(list);//ensure the integrity of the list structure
							selected = setupList(list, new Point(0,0), new Dimension(getRequiredWidth(list), getRequiredHeight(list)), toSelect);

							//perform a "selection" of the newly added action - display its properties	
							selectAction(selected);
	
							scroller.validate();
						} catch (Exception ex) {
							System.out.println("Unable to insert action in list");
							ex.printStackTrace();
						}

						repaint();
					}
					break;
			}
		}
	} 

	/**
	* The panel which displays the properties and property support of the currently selected action.
	* @author David Vivash
	* @version 1.0, 27/03/01
	*/
	private class PropertiesPanel extends JPanel {
		private PIYAction action = null;
		
		public PropertiesPanel() {
			setLayout(new BorderLayout());
		}
		
		/**
		* Each time an action is selected in the action list viewer, this method should be called
		* to update the properties.
		* @param action the action that has been selected
		*/
		public void setup(PIYAction action) {
			this.action = action;
			
			//get rid of the old properties panels
			removeAll();

			if (action != null) {
	
				//Set up the top panel which displays the name of the PIYAction on a yellow background
				JPanel top = new JPanel();
				top.setBackground(Color.yellow);
				top.setBorder(new LineBorder(Color.black));
				top.setMinimumSize(new Dimension(64,64));

				if (viewer != null) {
					ActionDescriptor descriptor = viewer.getDescriptor(action.getClass());
	
					top.add(new JLabel(descriptor.getName() + " Properties"));
					JPanel propPane = new JPanel();
					JScrollPane propertyScroller = new JScrollPane(propPane);
					add(top, BorderLayout.NORTH);
					add(propertyScroller, BorderLayout.CENTER);
							
					//Get the class support panels for this action
					PropertyManager propMan = new PropertyManager(PIY.getInstance().getClassSupport());
					ClassSupport[] panels = propMan.getClassSupport(action, PIYAction.class, false, null);
		
					//if any of the properties are of type Object, we need to find out what type they really are
					for (int i=0; i<panels.length; i++) {
						if (panels[i].getProperty().getType() == Object.class) {
							Property objectProperty = panels[i].getProperty();					
							try{
								Method bindingMethod = action.getClass().getMethod("get"+objectProperty.getName()+"Binding",null); 
			
								Property boundTo = (Property)bindingMethod.invoke(action, null);
		
								for (int j=0; j<panels.length; j++)
									if (boundTo.getName().equals(panels[j].getProperty().getName()))
										((ObjectSupport)panels[i]).setPairSupport(panels[j]);
									
							} catch (Exception e) {
								System.err.println("Error retrieving the binding method of " + objectProperty.getName());
							}
						}
					}
		
					//Layout the class support panels
					GridBagLayout gridbag = new GridBagLayout();
					GridBagConstraints c = new GridBagConstraints();
			
					propPane.setLayout(gridbag);
					c.fill = GridBagConstraints.HORIZONTAL;
					c.ipadx = 4; c.ipady = 8;
					c.insets = new Insets(4,4,4,4);
		
					for (int i=0; i<panels.length; i++) {
			
						c.gridy = i; c.gridx = 0; c.weightx = 0;
						JLabel label = new JLabel(panels[i].getProperty().getName());
						gridbag.setConstraints(label, c);
						propPane.add(label);
						
						c.gridx = 1; c.weightx = 1.0;
						gridbag.setConstraints(panels[i], c);
						propPane.add(panels[i]);
					}
		
		
					//See if the action returns a value
					Class returnType = action.getReturnType();
					//the return place is stored in the action list
					if (returnType != null) {
						ClassSupport returnSupport = new FixedPropertySupport(new Property(this, "Return", FixedProperty.class));
		
						c.gridy = panels.length; c.gridx = 0; c.weightx = 0;
						JPanel strings = new JPanel();
						strings.setLayout(new BorderLayout());
						strings.add(new JLabel("Store"), BorderLayout.NORTH);
						strings.add(new JLabel("result"), BorderLayout.CENTER);
						strings.add(new JLabel("in"), BorderLayout.SOUTH);
						gridbag.setConstraints(strings, c);
						propPane.add(strings);
						
						c.gridx = 1; c.weightx = 1.0;
						gridbag.setConstraints(returnSupport, c);
						propPane.add(returnSupport);
					}
		
					//add a blank panel at the bottom to make sure the (real) properties are 
					//aligned to the top of the frame.
					c.gridy = returnType != null ? panels.length+1 : panels.length;
					c.weighty = 1.0;
					JPanel blank = new JPanel();
					gridbag.setConstraints(blank, c);
					propPane.add(blank);
				}

			}	
			
			validate();
			repaint();
		}

		/**
		* This is set up to support the return sent from the action.
		* @return the property fixed to the type of the getReturnType() of the action
		*/
		public FixedProperty getReturn() {
			Property val = list.getReturn(action);
			FixedProperty prop = new FixedProperty(null, action.getReturnType());
			if (val == null) val = new Property(null, "", action.getReturnType());
			prop.setProperty(val);
			prop.setDynamicOnly(true);
			setReturn(prop);
			return prop;
		}
		
		/**
		* Set where the action should send its return value to.
		* @param property the property fixed to the type that the action returns
		*/
		public void setReturn(FixedProperty property) {
			list.setReturn(action, property.getProperty());
		}
	}

	/**
	* Supports the selection and renaming of action lists.
	* @author David Vivash
	* @version 1.0, 27/03/01
	*/
	private class ActionListNamePanel extends JPanel implements ActionListener {

		private ActionList list = null;
		private JComboBox combo = new JComboBox();
		private static final String DELETE_LIST = "deleteList";

		private boolean editting = false;  //when this is true, it mean the action list combo box is being updated,
											//so when it fires the actionPerformed method, it shouldn't do anything 

		public ActionListNamePanel(ActionList list) {
			setup(list);
		}

		/**
		* Sets up the panel view for the specified action list.
		* @param list the list to initially display as being selected
		*/
		private void setup(ActionList list) {
			removeAll();
			combo = new JComboBox(ProjectHandler.getInstance().getProject().getActionListNamer().getNames());
			
			combo.insertItemAt("", 0);
			
			setLayout(new GridLayout(2,1));
			JPanel comboPanel = new JPanel();
			comboPanel.setLayout(new BorderLayout());
			comboPanel.add(combo, BorderLayout.CENTER);
			
			String sep = File.separator;
			String images = "piy" + sep + "images" + sep;

			FloatingButton delActionList = new FloatingButton(new ImageIcon(images + "delete.gif").getImage(), DELETE_LIST, false);
			delActionList.addActionListener(this);
			delActionList.setToolTipText("Delete Action List");
			delActionList.setPreferredSize(new Dimension(24,24));

			comboPanel.add(delActionList, BorderLayout.WEST);
			
			add(comboPanel);

			this.list = list;

			if (list != null) {
				Property listName = new Property(this, "Name", String.class);

				JPanel namePanel = new JPanel();
				namePanel.setLayout(new BorderLayout());
				namePanel.add(new JLabel(" Name   "), BorderLayout.WEST);
				ClassSupport panel = new StringSupport(listName);
				namePanel.add(panel, BorderLayout.CENTER);

				add(namePanel);
				combo.setSelectedItem(ProjectHandler.getInstance().getProject().getActionListNamer().getName(list));
			} else {
				combo.setSelectedIndex(0); //select the blank item at the top of the list
			}

			combo.addActionListener(this);

			validate();
			repaint();
		}

		/**
		* Creates a new actionlist which is automatically added to the project currently being editted.
		* Its name is also automatically added to the list of actionlist names in the combo box.
		* @return the ActionList object that was created
		*/
		public ActionList newList() {
			ActionList list = new ActionList();
			ProjectHandler.getInstance().getProject().getActionListNamer().add(list, "ActionList");
			setup(list);
			return list;
		}

		/**
		* Sets the name of the action list currently being editted.
		* @param name the new name for the action list.  The name will not be set unless it is 
		* unique in the project context.
		*/
		public void setName(String name) {
			editting = true;
			PIYNamer namer = ProjectHandler.getInstance().getProject().getActionListNamer();
			String oldName = namer.getName(list);
			namer.renameValue(oldName, name);
			String newName = namer.getName(list);
			combo.removeItem(oldName);
			combo.addItem(newName);
			combo.setSelectedItem(newName);
			editting = false;
		}

		/**
		* Gets the name of the action list currently being editted.
		* @returns the unique name for the action list.
		*/
		public String getName() {
			return ProjectHandler.getInstance().getProject().getName(list);
		}


		public Dimension getPreferredSize() { return new Dimension(100,48); }
		public Dimension getMinimumSize() { return new Dimension(100,48); }
		public Dimension getMaximumSize() { return new Dimension(100,48); }

		//------------- ActionListener Method ---------------
		public void actionPerformed(ActionEvent e) {
			if (!editting) {
				if (e.getActionCommand() == DELETE_LIST) {
					viewer.deleteList();			
					ProjectHandler.getInstance().projectChanged();
					list = null;
				} else {				
					String listName = (String)combo.getSelectedItem();
					if (listName.equals("")) list = null;
					else list = (ActionList)ProjectHandler.getInstance().getProject().getActionListNamer().getValue(listName);
				}
				
				viewer.setList(list);
				setup(list);
			} 
		}
	}

}