package piy;

import java.io.File;
import javax.swing.ImageIcon;

/**
* Describes the Null comparator action.  It has restricted instantiation via the 
* Singleton pattern for no other reason than it makes no sense to have
* more than one of these.
* @author David Vivash
* @version 1.0, 30/04/01
*/
public class NullComparatorDescriptor extends ActionDescriptor
{
	private static NullComparatorDescriptor instance = null;

	private NullComparatorDescriptor() { }

	public Class getDescribedClass() { return NullComparator.class; }

	public String getName() { return "Null"; }

	public String getDescription() { return "Does nothing"; }

	public static NullComparatorDescriptor getInstance() {
		return (instance == null) ? instance = new NullComparatorDescriptor() : instance;
	}

	/**
	* Returns the (40x40) icon that should be displayed in the actionlist viewer window.
	* @return the (large) image of the null action
	*/
	public ImageIcon getBigIcon() {
		String sep = File.separator;
		return new ImageIcon("piy" + sep + "images" + sep + "nullaction.gif");
	}

}