package piy;

/**
* Representation of an abstract action.  All actions should be implemented into PIY by
* firstly implementing this class, and secondly by being placed in the "action" directory.
* Any properties of the action that need to be modifiable by the user should be named in
* accordance to the PIY naming rules for properties.
* @author David Vivash
* @version 1.0.1, 22/03/01
*/
public interface PIYAction
{
	/**
	* Get the return type of this action.  
	* @return the class representing the return type of the execute method. May be null 
	* if the action doesn't return anything.
	*/
	public Class getReturnType();

	/**
	* Run the action.
	* @return the result of running the action, whose type should be the same as that 
	* returned by getReturnType()
	*/
	public Object execute();
}