package piy.support;

import javax.swing.*;
import java.awt.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.event.*;
import piy.*;

import java.io.File;
import java.util.*;

/**
* Frame in which user can edit boolean expressions.  Relies on PIY for comparator descriptors.
* <p>
* Muse: Should this be made a singleton?
* <p>
* This class is suspiciously similar to ActionListEditor - REFACTOR!
* @author David Vivash
* @version 1.0, 30/04/01
*/
public class BooleanExpressionEditor extends JFrame implements ActionListener, ChangeListener {

	private static final String OR_BEFORE	= "orBefore";
	private static final String OR_AFTER	= "orAfter";
	private static final String AND_BEFORE	= "andBefore";
	private static final String AND_AFTER	= "andAfter";

	private static final String DELETE		= "delete";
	private static final String SWAP		= "swap";


	private Color selectionColor = new Color(190, 170, 250);

	private BooleanComparable rootExpression = null;

	//The action descriptor that has currently been selected on the toolbar
	private ActionDescriptor toolbarSelection = null;

	private JScrollPane scroller = null;
	private ExpressionViewer viewer = null;
	private ScrollingButtonPanel[] comparatorPanel 	= null;
	private JTabbedPane tabPane 					= null;
	private PropertiesPanel propertiesPanel 		= new PropertiesPanel();
	private BooleanExpressionEditor editor			= null;

	private ExpressionIcon selected = null;
	private ExpressionIcon rootIcon = null;

	private static final int ADD = 0;
	private static final int SELECT = 1;

	private int editType = SELECT;
	
	private Property whereBound = null;

	private ActionDescriptor[] comparatorDescriptors = null;

	public BooleanExpressionEditor(BooleanComparable comparable, Property whereBound) {
		super("Boolean Expression Editor");

		rootExpression = comparable;
		comparatorDescriptors = PIY.getInstance().getComparators();
		editor = this;
		this.whereBound = whereBound;
		
		
		getContentPane().setLayout(new BorderLayout());

		JPanel topPanel = new JPanel() {
			public void paint(Graphics g) {
				super.paint(g);
				g.setColor(getBackground().darker());
				g.drawLine(60,2,60,getHeight()-8);
				g.setColor(getBackground().brighter());
				g.drawLine(61,2,61,getHeight()-8);
			}		
		};
		
		topPanel.setLayout(new BorderLayout());

		//---- Set up toolbar buttons -------
		JPanel toolbarButtons = new JPanel();

		toolbarButtons.setLayout(null);
		toolbarButtons.setPreferredSize(new Dimension(120,60));
		
		String sep = File.separator;
		String images = "piy" + sep + "images" + sep;

		FloatingButton andBefore = new FloatingButton(new ImageIcon(images + "insertbefore.gif").getImage(), AND_BEFORE, false);
		andBefore.addActionListener(this);
		andBefore.setBounds(4, 6, 22, 22);
		andBefore.setToolTipText("Insert AND before");

		FloatingButton andAfter = new FloatingButton(new ImageIcon(images + "insertafter.gif").getImage(), AND_AFTER, false);
		andAfter.addActionListener(this);
		andAfter.setBounds(30, 6, 22, 22);
		andAfter.setToolTipText("Insert AND after");

		FloatingButton orBefore = new FloatingButton(new ImageIcon(images + "orbefore.gif").getImage(), OR_BEFORE, false);
		orBefore.addActionListener(this);
		orBefore.setBounds(4, 34, 22, 22);
		orBefore.setToolTipText("Insert OR above");

		FloatingButton orAfter = new FloatingButton(new ImageIcon(images + "orafter.gif").getImage(), OR_AFTER, false);
		orAfter.addActionListener(this);
		orAfter.setBounds(30, 34, 22, 22);
		orAfter.setToolTipText("Insert OR below");

		FloatingButton delete = new FloatingButton(new ImageIcon(images + "bin.gif").getImage(), DELETE, false);
		delete.addActionListener(this);
		delete.setBounds(64, 20, 22, 22);
		delete.setToolTipText("Delete Selected");

		FloatingButton swap = new FloatingButton(new ImageIcon(images + "swap.gif").getImage(), SWAP, false);
		swap.addActionListener(this);
		swap.setBounds(90, 20, 22, 22);
		swap.setToolTipText("Swap Order");

		toolbarButtons.add(andBefore);
		toolbarButtons.add(andAfter);
		toolbarButtons.add(orBefore);
		toolbarButtons.add(orAfter);

		toolbarButtons.add(delete);
		toolbarButtons.add(swap);


		//------ Set up comparator buttons panel -----
		tabPane = new JTabbedPane();
		ArrayList collections = sortCategories();
		
		comparatorPanel = new ScrollingButtonPanel[collections.size()];

		FloatingButton toolButton		= null;
		ActionDescriptor[] descriptors	= null;
		
		for (int i=0; i<collections.size(); i++) {
			descriptors = (ActionDescriptor[])collections.get(i);
			comparatorPanel[i] = new ScrollingButtonPanel(descriptors.length, 26, 26, 4);
			tabPane.addTab(descriptors[0].getCategory(), comparatorPanel[i]);
			comparatorPanel[i].addChangeListener(this);

			for (int j=0; j<descriptors.length; j++) {
				comparatorPanel[i].addButton(toolButton = new FloatingButton(descriptors[j].getIcon(), ""+j, true));

				//where the action descriptor is located in the actionDescriptors array
				int num = findDescriptor(descriptors[j]);
				toolButton.setIdentifier(num);

				toolButton.setToolTipText(descriptors[j].getName());
			}
		}
		
		//Move the "General" tab to be the first tab in the pane
		int posOfGeneral = tabPane.indexOfTab("General");
		if (posOfGeneral != -1) {
			if (posOfGeneral != 0) { 
				Component generalPanel = tabPane.getComponentAt(posOfGeneral);
				tabPane.removeTabAt(posOfGeneral);
				tabPane.insertTab("General", null, generalPanel, null, 0);
			}
		}
		
		
		topPanel.add(toolbarButtons, BorderLayout.WEST);
		topPanel.add(tabPane, BorderLayout.CENTER);

		JPanel padding = new JPanel();
		padding.setPreferredSize(new Dimension(0,8));
		topPanel.add(padding, BorderLayout.SOUTH);

//		propertiesPanel = new JPanel();
		propertiesPanel.setPreferredSize(new Dimension(180,100));
		propertiesPanel.setMaximumSize(new Dimension(400,100));
		propertiesPanel.setMinimumSize(new Dimension(50,100));

		viewer = new ExpressionViewer();
		scroller = new JScrollPane(viewer);

		JSplitPane bottomPanel = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, propertiesPanel, scroller);

		propertiesPanel.setPreferredSize(new Dimension(180,100));

		getContentPane().add(topPanel, BorderLayout.NORTH);
		getContentPane().add(bottomPanel, BorderLayout.CENTER);

		setSize(400,300);
	}
		
	/**
	* Find the specified Descriptor's location in the comparatorDescriptors array
	* @param descriptor the descriptor to search for
	* @return the descriptor's location in the comparatorDescriptors array, or -1 if it was not found
	*/
	private int findDescriptor(ActionDescriptor descriptor) {
		for (int i=0; i<comparatorDescriptors.length; i++) 
			if (descriptor == comparatorDescriptors[i])
				return i;

		return -1;
	}			
	
	/**
	* Retrieve the ActionDescriptor for a particular comparator class type.
	* @param comparator the class of the comparator whose action descriptor is required
	* @return the action descriptor of the specified comparator class, or null if an
	* an appropriate descriptor was not found
	*/
	protected ActionDescriptor getDescriptor(Class comparator) {
		//The NullAction will not be present in the descriptor array
		if (comparator == NullComparator.class) return NullComparatorDescriptor.getInstance();

		for (int i=0; i<comparatorDescriptors.length; i++)
			if (comparatorDescriptors[i].getDescribedClass() == comparator)
				return comparatorDescriptors[i];
	
		return null;
	}

	/**
	* Selects the specified icon, deselcting the last one.
	* @param icon the ExpressionIcon to select
	*/
	protected void select(ExpressionIcon icon) {

		if (selected != null) selected.setBackground(Color.white);
		selected = icon;
		if (selected != null) selected.setBackground(selectionColor);
		viewer.repaint();
		
		propertiesPanel.setup(icon == null ? null : icon.getComparable());
	}


	//----- ChangeListener method ------------
	private boolean editting = false;
	public void stateChanged(ChangeEvent e) {
		if (!editting) {
			ScrollingButtonPanel inside = (ScrollingButtonPanel)tabPane.getSelectedComponent();
			int pos = inside.getSelectedIndex();
		
			editting = true;
			//Clear selections on other tabs now
			for (int i=0; i<comparatorPanel.length; i++) 
				if (comparatorPanel[i] != inside) comparatorPanel[i].clearSelection();
			
			//TO DO!!!! Get the action descriptor that was selected
			if (pos == -1) {
				toolbarSelection = null;
			} else {
				FloatingButton button = (FloatingButton)inside.getSelectedButton();
				toolbarSelection = comparatorDescriptors[button.getIdentifier()];
			}

			
			editting = false;
			
			if (pos == -1) editType = SELECT;
			else editType = ADD;
		}
	}

	
	/**
	* Takes the comparator descriptors, and sorts them into a collection of arrays - each array contains
	* descriptors of comparators in the same category.  So sorts out all of the string comparators from 
	* the files comparators, for example.  Each entry in the array list returned is an ActionDescriptor[].
	* <p>
	* <b>This method needs refactoring.</b> This method is the same (virtually) as the sortCategories
	* method in the ActionListEditor.  Maybe it could be moved to a PIYUtilities class.
	* @return a collection of ActionDescriptor arrays, with each array containing descriptors in the
	* same category.
	*/
	private ArrayList sortCategories() {
		//generate a map from each category name to the number of actions in that category
		//ie categoryName(String) -> Count(Inteeger)
		HashMap nameToCount = new HashMap(comparatorDescriptors.length);
		Integer num = null;
	
		for (int i=0; i<comparatorDescriptors.length; i++) {
			num = (Integer)nameToCount.get(comparatorDescriptors[i].getCategory());
		
			if (num == null) //category name doesn't yet exist
				num = new Integer(1);
			else 
				num = new Integer(num.intValue() + 1);
			
			nameToCount.put(comparatorDescriptors[i].getCategory(), num);
		}
	
		//Now construct each array and store it in the arraylist to return.
		//We could return an array of ActionDescriptor arrays, but I think that's a bit
		//too confusing
		ArrayList toReturn = new ArrayList(nameToCount.size());
		ActionDescriptor[] array = null;
		String currentName = null;
		int arrayCounter = 0;
	
		//Iterate over each named category
		Iterator nameIterator = nameToCount.keySet().iterator();
		while (nameIterator.hasNext()) {
			currentName = (String)nameIterator.next();
			
			array = new ActionDescriptor[((Integer)nameToCount.get(currentName)).intValue()];
			arrayCounter = 0;
			for (int j=0; j<comparatorDescriptors.length; j++) {
				if (comparatorDescriptors[j].getCategory().equals(currentName)) {
					array[arrayCounter] = comparatorDescriptors[j];
					arrayCounter++;
				}
			}
			
			toReturn.add(array);
		}
		
		return toReturn;
	}
	
	public boolean getNegate() { return selected == null ? false : selected.getComparable().getNegate(); }
	public void setNegate(boolean negate) {
		if (selected == null) return;		
		selected.getComparable().setNegate(negate);
		negate(selected);
	}
	protected void negate(ExpressionIcon icon) {
		icon.setBorder(new LineBorder(icon.getComparable().getNegate() ? Color.red : Color.black, 1));
	}



	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		
		if (command == AND_BEFORE || command == OR_BEFORE) {
			ExpressionIcon parentIcon = selected.getParentIcon();

			if (parentIcon == null) {
				rootExpression = new BooleanExpression();
				((BooleanExpression)rootExpression).setFirst(new NullComparator());
				((BooleanExpression)rootExpression).setSecond(selected.getComparable());
				whereBound.setValue(rootExpression);
				if (command == OR_BEFORE) ((BooleanExpression)rootExpression).setAnd(false);
				rootIcon.setup(rootExpression, null);
			} else {
				BooleanExpression ex = new BooleanExpression();
				ex.setFirst(new NullComparator());
				ex.setSecond(selected.getComparable());				

				if (command == OR_BEFORE) ex.setAnd(false);
			
				if (((BooleanExpression)parentIcon.getComparable()).getFirst() == selected.getComparable()) ((BooleanExpression)parentIcon.getComparable()).setFirst(ex);
				else ((BooleanExpression)parentIcon.getComparable()).setSecond(ex);
				rootIcon.setup(rootExpression, null);					
			}	
		} else if (command == AND_AFTER || command == OR_AFTER) {
			ExpressionIcon parentIcon = selected.getParentIcon();
			if (parentIcon == null) {
				rootExpression = new BooleanExpression();
				((BooleanExpression)rootExpression).setSecond(new NullComparator());
				((BooleanExpression)rootExpression).setFirst(selected.getComparable());
				whereBound.setValue(rootExpression);
				if (command == OR_AFTER) ((BooleanExpression)rootExpression).setAnd(false);
				rootIcon.setup(rootExpression, null);
			} else {
				BooleanExpression ex = new BooleanExpression();
				ex.setSecond(new NullComparator());
				ex.setFirst(selected.getComparable());				
			
				if (command == OR_AFTER) ex.setAnd(false);

				if (((BooleanExpression)parentIcon.getComparable()).getFirst() == selected.getComparable()) ((BooleanExpression)parentIcon.getComparable()).setFirst(ex);
				else ((BooleanExpression)parentIcon.getComparable()).setSecond(ex);
				rootIcon.setup(rootExpression, null);					
			}			
		} else if (command == SWAP) {
			if ((selected != null) && (selected.getComparable() instanceof BooleanExpression)) {
				BooleanExpression toSwap = (BooleanExpression)selected.getComparable();
				if (toSwap instanceof BooleanExpression) {
					BooleanComparable temp = toSwap.getFirst();
					toSwap.setFirst(toSwap.getSecond());
					toSwap.setSecond(temp);
					selected.setup(selected.getComparable(), selected.getParentIcon());
				}
			}

		} else if (command == DELETE) {
			if (selected != null) {
				
				if (selected.getParentIcon() == null) { //top level icon
					rootExpression = new NullComparator();
					whereBound.setValue(rootExpression);
					rootIcon.setup(rootExpression, null);
				} else {
					ExpressionIcon oldParent = selected.getParentIcon();
					BooleanComparable toBecome = null;
					
					if ( ((BooleanExpression)oldParent.getComparable()).getFirst() == selected.getComparable()) {
						toBecome = ((BooleanExpression)oldParent.getComparable()).getSecond();
					} else {
						toBecome = ((BooleanExpression)oldParent.getComparable()).getFirst();					
					}
					
					ExpressionIcon oldParentParent = oldParent.getParentIcon();

					if (oldParentParent == null) { //top level
						rootExpression = toBecome;
						whereBound.setValue(rootExpression);
						rootIcon.setup(rootExpression, null);
					} else {
						if ( ((BooleanExpression)oldParentParent.getComparable()).getFirst() == oldParent.getComparable()) {
							((BooleanExpression)oldParentParent.getComparable()).setFirst(toBecome);
						} else {
							((BooleanExpression)oldParentParent.getComparable()).setSecond(toBecome);
						}

						rootIcon.setup(rootExpression, null);
					}
				}
				
				select(null);
			}		
		}
	}

	/**
	* These icons provide the mechanism by which individual comparators can be selected in
	* the expression.  The size required by one of these is determined by the size of
	* the sub-expressions.  There is generally one top level one of these, which contains 
	* more internally.
	* @author David Vivash
	* @version 1.0, 30/04/01
	*/
	private class ExpressionIcon extends JPanel implements MouseListener {
		private BooleanComparable expression	= null;
		private ExpressionIcon parentIcon		= null;
		private ImageIcon icon					= null;
	
		//coords for the line joining two components in the expression
		private int lineX = 0, lineY = 0;

		private static final int xGap = 48, yGap = 32, xPadding = 6, yPadding = 6, xSize=48, ySize=48;
	
		public ExpressionIcon() {
			setBackground(Color.white);
			setLayout(null);
			addMouseListener(this);				
		}

		/**
		* Constructs a new expression icon.  If the supplied comparator is a boolean expression,
		* this constructor will add new expression icons internally to deal with it.
		* @param ex the comparator to construct an icon for.  If null, this constructor throws
		* an Error - to create NullComparators, use the NullComparator object instead
		* @param parentIcon the expression icon that contains this one.  A top level icon
		* should set this parameter to null
		*/
		public void setup(BooleanComparable ex, ExpressionIcon parentIcon) {
			if (ex == null) throw new Error("Comparator was null");

			removeAll();

			this.expression = ex;
			this.parentIcon = parentIcon;
			
			if (ex instanceof BooleanExpression) {
				BooleanExpression boolEx = (BooleanExpression)ex;
				ExpressionIcon first = new ExpressionIcon();
				first.setup(boolEx.getFirst(), this);
				ExpressionIcon second = new ExpressionIcon();
				second.setup(boolEx.getSecond(), this);

				add(first);
				add(second);
				
				if (boolEx.getAnd()) {
					Dimension size1 = getRequiredSize(first.getComparable());
					Dimension size2 = getRequiredSize(second.getComparable());
					int maxHeight = size1.height > size2.height ? size1.height : size2.height;					
	
					first.setLocation(new Point(xPadding, yPadding + (maxHeight - size1.height)/2));
					second.setLocation(new Point(size1.width + xPadding+xGap, yPadding + (maxHeight - size2.height)/2));

					lineX = size1.width + xPadding;
					lineY = yPadding + maxHeight/2;
				} else {
					Dimension size1 = getRequiredSize(first.getComparable());
					Dimension size2 = getRequiredSize(second.getComparable());
					int maxWidth = size1.width > size2.width ? size1.width : size2.width;
	
					first.setLocation(new Point(xPadding + (maxWidth - size1.width)/2, yPadding));
					second.setLocation(new Point(xPadding + (maxWidth - size2.width)/2, yPadding + size1.height + yGap));

					lineX = xPadding + maxWidth/2;
					lineY = size1.height + yPadding;
				}
																			
			} else if (ex instanceof BooleanComparable) {
				icon = editor.getDescriptor(ex.getClass()).getBigIcon();
							}
			
			editor.negate(this);

			this.setSize(getRequiredSize(ex));
			if (scroller != null) scroller.validate();
		}
	
		public BooleanComparable getComparable() { return expression; }		
		public ExpressionIcon getParentIcon() {	return parentIcon; }
		
		public Dimension getRequiredSize(BooleanComparable comp) {
			if (comp == null) return new Dimension(xSize, ySize);

			if (comp instanceof PIYComparator) return new Dimension(xSize, ySize);
			
			if (comp instanceof BooleanExpression) {

				BooleanExpression boolEx = (BooleanExpression)comp;
				
				boolean leftToRight = boolEx.getAnd(); //we display ANDs left to right, ORs top to bottom
				
				Dimension size1 = getRequiredSize(boolEx.getFirst());
				Dimension size2 = getRequiredSize(boolEx.getSecond());
				int maxHeight = size1.height > size2.height ? size1.height : size2.height;
				int maxWidth = size1.width > size2.width ? size1.width : size2.width;

				if (leftToRight) {			
					return new Dimension(size1.width + size2.width + xPadding*2 + xGap,
										maxHeight + yPadding*2);
				} else {				
					return new Dimension(maxWidth + xPadding*2,
										size1.height + size2.height + yPadding*2 + yGap);
				}
			}
		
			return new Dimension(0, 0);
		}

		public Dimension getPreferredSize() { return getRequiredSize(expression); }

		/**
		* Paints the icon - it is either an expression (where it joins two comparators together)
		* or it is a comparator (where it draws the icon).
		* @param g the graphics context on which to draw
		*/		
		public void paint(Graphics g) {
			super.paint(g);
			
			if (expression instanceof BooleanExpression) {
				//draw lines
				if ( ((BooleanExpression)expression).getAnd() ) {
					g.drawLine(lineX, lineY, xGap + lineX, lineY);
					g.drawString("AND", lineX + xGap/2 - g.getFontMetrics().stringWidth("AND")/2, lineY - 2);
				} else {
					g.drawLine(lineX, lineY, lineX, lineY + yGap);
					g.drawString("OR", lineX + 4, lineY + yGap/2 + g.getFontMetrics().getHeight()/2 - 4);
				}

			} else if (expression instanceof BooleanComparable) {
				g.drawImage(icon.getImage(), getWidth()/2-icon.getIconWidth()/2, getHeight()/2-icon.getIconHeight()/2, this);
			}
		
		}
	
		public void mousePressed(MouseEvent e) { }
		public void mouseReleased(MouseEvent e) { }
		public void mouseClicked(MouseEvent e) { 
			switch (editType) {
				case SELECT:
					editor.select(this);
					break;
				case ADD:
					if (expression instanceof NullComparator) {
						//add the selected comparator to the expression
						try{					
							PIYComparator toSelect = (PIYComparator)toolbarSelection.getDescribedClass().newInstance();

							if (parentIcon == null) { //top level icon
								whereBound.setValue(toSelect);
								setup(toSelect, null);
							} else {
							
								BooleanExpression parent = (BooleanExpression)parentIcon.getComparable();

								if (parent.getFirst() == expression) parent.setFirst(toSelect);
								else parent.setSecond(toSelect);
								setup(toSelect, parentIcon);
							}

							for (int i=0; i<comparatorPanel.length; i++)
								comparatorPanel[i].clearSelection(); //deselect the icon on the toolbar

							scroller.validate();

						} catch (Exception X) {
							System.out.println("Unable to insert comparator into expression");
							X.printStackTrace();
						}

						repaint();

						editor.select(this);
					}
					break;
			}
		}
		public void mouseEntered(MouseEvent e) { }
		public void mouseExited(MouseEvent e) { }
	
	}

	private class ExpressionViewer extends JPanel implements MouseListener {
		public ExpressionViewer() {
			super();	
			setBackground(Color.white);
			setLayout(null);
			ExpressionIcon icon = new ExpressionIcon();
			icon.setup(rootExpression, null);			
			add(icon);			
			addMouseListener(this);
			
			rootIcon = icon;
		}	

		public Dimension getPreferredSize() { return rootIcon.getPreferredSize(); }

		//------- Mouse Listener methods ----------
		public void mousePressed(MouseEvent e) { }
		public void mouseReleased(MouseEvent e) { }
		public void mouseEntered(MouseEvent e) { }
		public void mouseExited(MouseEvent e) { }
		public void mouseClicked(MouseEvent e) {
			editor.select(null); //the user hasn't clicked on any of the internal icons, so clear the selection
		}
	}
	
	/**
	* The panel which displays the properties and property support of the currently selected comparator.
	* @author David Vivash
	* @version 1.0, 01/05/01
	*/
	private class PropertiesPanel extends JPanel implements ActionListener {
		private BooleanComparable comparable = null;
		private JComboBox combo = null;
		
		public PropertiesPanel() {
			setLayout(new BorderLayout());
		}
		
		/**
		* Each time an action is selected in the action list viewer, this method should be called
		* to update the properties.
		* @param action the action that has been selected
		*/
		public void setup(BooleanComparable comparable) {
			this.comparable = comparable;
			
			//get rid of the old properties panels
			removeAll();

			if ((comparable != null) && !(comparable instanceof NullComparator)) {
	
				//Set up the top panel which displays the name of the PIYAction on a yellow background
				JPanel top = new JPanel();
				top.setBackground(Color.yellow);
				top.setBorder(new LineBorder(Color.black));
				top.setMinimumSize(new Dimension(64,64));
				ActionDescriptor descriptor = getDescriptor(comparable.getClass());
				if (descriptor != null) top.add(new JLabel(descriptor.getName() + " Properties"));
				else top.add(new JLabel("Comparator Properties"));
				JPanel propPane = new JPanel();
				JScrollPane propertyScroller = new JScrollPane(propPane);
				add(top, BorderLayout.NORTH);
				add(propertyScroller, BorderLayout.CENTER);
	
				//Get the class support panels for this action
				//Layout the class support panels
				GridBagLayout gridbag = new GridBagLayout();
				GridBagConstraints c = new GridBagConstraints();
		
				propPane.setLayout(gridbag);

				c.fill = GridBagConstraints.HORIZONTAL;
				c.ipadx = 4; c.ipady = 8;
				c.insets = new Insets(4,4,4,4);
	

				ClassSupport negateSupport = new BoolSupport(new Property(editor, "Negate", boolean.class));

				JLabel label = new JLabel("Negate?");
				JPanel negatePanel = new JPanel();
				negatePanel.setLayout(new BorderLayout());
				negatePanel.add(label, BorderLayout.WEST);
				negatePanel.add(negateSupport, BorderLayout.CENTER);
				
				c.gridx = 0; c.weightx = 1.0;
				gridbag.setConstraints(negatePanel, c);
				propPane.add(negatePanel);


				int ypos = 1;

				if (comparable instanceof PIYComparator) {
					PIYComparator comp = (PIYComparator)comparable;
				
					//add support for first
					Class firstType = comp.getFirstType();
					if ( (firstType != null) && (firstType != void.class)) {
						FixedPropertySupport firstProperty = new FixedPropertySupport(new Property(comp, "First", firstType));					
						c.gridy = ypos; c.gridx = 0;
						gridbag.setConstraints(firstProperty, c);
						propPane.add(firstProperty);
					
						++ypos;
					}
					
					//Add combo box here...
					combo = new JComboBox(comp.getModes());
					combo.setSelectedItem(comp.getMode());
					c.gridy = ypos; c.gridx = 0;
					combo.setPreferredSize(new Dimension(0, 24));
					gridbag.setConstraints(combo, c);
					propPane.add(combo);

					combo.addActionListener(this);
									
					++ypos;
							
					//now add support for second
					Class secondType = comp.getSecondType();
					if ( (firstType != null) && (firstType != void.class)) {
						FixedPropertySupport secondProperty = new FixedPropertySupport(new Property(comp, "Second", secondType));
						c.gridy = ypos; c.gridx = 0;
						gridbag.setConstraints(secondProperty, c);
						propPane.add(secondProperty);
					
						++ypos;
					}
				}
			
				//add a blank panel at the bottom to make sure the (real) properties are 
				//aligned to the top of the frame.
				c.gridy = ypos; c.weighty = 1.0;
				JPanel blank = new JPanel();
				gridbag.setConstraints(blank, c);
				propPane.add(blank);
			}
			
			
			validate();
			repaint();
		}

		private ActionDescriptor getDescriptor(Class type) {
			for (int i=0; i<comparatorDescriptors.length; i++)
				if (comparatorDescriptors[i].getDescribedClass() == type) return comparatorDescriptors[i];


			return null;		
		}

		public void actionPerformed(ActionEvent e) {
			((PIYComparator)comparable).setMode((String)combo.getSelectedItem());
		
		}

	}
}
