package piytest;

import junit.framework.*;
import piy.*;

import piytest.support.GenericPropertyHolder;

/**
* Test set for the ActionList class.
* @author David Vivash
* @version 1.0, 26/04/01
*/
public class ActionListTest extends TestCase
{
	private ActionList emptyList	= null;
	private ActionList testList		= null;
	
	public ActionListTest(String name) {
		super(name);
	}

	public static Test suite() {
		return new TestSuite(ActionListTest.class);
	}

	public void setUp()
	{
		emptyList = new ActionList();
		
		testList = new ActionList();
		testList.add(new MinimalAction());
		testList.add(new MinimalAction());
		testList.add(new MinimalAction());
		testList.add(new MinimalAction());
		testList.add(new MinimalAction());
	}

	/**
	* Also tests size and get(pos).
	*/
	public void testAddAction() {
		int size = emptyList.size();
		emptyList.add(new MinimalAction());
		assert(emptyList.size() > size);
		
		int size2 = testList.size();
		testList.add(new MinimalAction());
		assert(testList.size() > size);
		
		
		PIYAction action = new MinimalAction();
		emptyList.add(action);
		assert(emptyList.get(emptyList.size()-1) == action);
	}
	
	public void testSize() {
		int size = testList.size();	
		testList.add(new MinimalAction());
		assert(testList.size() == size+1);
	}
	
	public void testGetIndex() {
		PIYAction toGet = new MinimalAction();
		int size = testList.size();
		testList.add(toGet);
		
		assert(testList.getIndex(toGet) == size);
		
		testList.add(new MinimalAction());
		
		assert(testList.getIndex(toGet) == size);
	}
	
	public void testGet() {
		PIYAction toGet = new MinimalAction();
		int size = testList.size();
		testList.add(toGet);
		
		assert(testList.get(size) == toGet);

		testList.add(new MinimalAction());
		
		assert(testList.get(size) == toGet);		
	
	}
	
	public void testRemoveAction() {
		PIYAction toGet = new MinimalAction();
		int size = testList.size();
		testList.add(toGet);
		
		testList.removeAction(toGet);

		assert(testList.size() == size);
		assert(testList.getIndex(toGet) == -1);

		assert(emptyList.removeAction(toGet) == -1);
	}
	
	public void testReturn() {
		GenericPropertyHolder holder = new GenericPropertyHolder();
		Property property = new Property(holder, "StringValue", String.class);
		
		PIYAction toSet = new MinimalAction();
		testList.add(toSet);
		
		testList.setReturn(toSet, property);
		assert(testList.getReturn(toSet) == property);
		
		assert(testList.getReturn(new MinimalAction()) == null);
		assert(emptyList.getReturn(new MinimalAction()) == null);
		assert(emptyList.getReturn(toSet) == null);
	
		//make sure that removing the action from the list removes the action->return mapping
		testList.removeAction(toSet);
		assert(testList.getReturn(toSet) == null);
	}
	
	private class MinimalAction implements PIYAction {
		public Class getReturnType() { return Void.class; }
		public Object execute() { return null; }
	}
}