package piytest;

import junit.framework.*;
import piy.Mapper;

/**
* Test set for the Mapper class.
* @author David Vivash
* @version 1.0, 22/11/00
*/
public class MapperTest extends TestCase
{
	private Mapper empty, full;
	private Integer mappedTo = new Integer(1);

	private Mapper mappedReference = new Mapper(1);

	public MapperTest(String name) {
		super(name);
	}

	public static Test suite() {
		return new TestSuite(MapperTest.class);
	}

	public void setUp()
	{
		empty = new Mapper(4);
		full = new Mapper(4);
		full.put("zero", mappedReference);
		full.put("one", mappedTo);
		full.put("two", new Integer(2));
		full.put("three", new Integer(3));
		full.put("four", new Integer(4));
	}

	public void testAddMapping() {
		assert(empty.put("one", new Integer(1)) == null);
		assert(full.put("five", new Integer(5)) == null);
		assert(full.getValue("five").equals(new Integer(5)));
	}

	public void testReAssignMapping() {
		assert(full.put("four", new Integer(8)).equals(new Integer(4)));
		assert(full.getValue("four").equals(new Integer(8)));
	}

	public void testGetValue() {
		assert(full.getValue("one").equals(new Integer(1)));
		assert(full.getValue("five") == null);
		assert(empty.getValue("anything") == null);
	}

	public void testGetKey() {
		assert(full.getKey(mappedTo).equals("one"));
		assert(full.getKey(new Mapper(1)) == null);
		assert(empty.getKey(new Integer(1)) == null);
	}

	public void testGetKeys() {
		assert(empty.keys().size() == 0);
		assert(full.keys().size() == 5);
		assert(full.keys().contains("four"));
	}

	public void testGetValues() {
		assert(empty.values().size() == 0);
		assert(full.values().size() == 5);
		assert(full.values().contains(new Integer(1)));
	}

	public void testRemoveKey() {
		assert(full.removeKey("one").equals(new Integer(1)));
		assert(full.getValue("one") == null);
		assert(full.keys().size() == full.values().size());
		assert(full.keys().size() == 4);

		assert(full.removeKey("one") == null);
		assert(full.values().size() == 4);

		assert(empty.removeKey("one") == null);
	}

	public void testRemoveValue() {
		assert(full.removeValue(new Integer(1)).equals("one"));
		assert(full.getValue("one") == null);
		assert(full.getKey(new Integer(1)) == null);

		assert(full.keys().size() == full.values().size());
		assert(full.keys().size() == 4);

		assert(full.removeValue(new Integer(1)) == null);
		assert(full.values().size() == 4);

		assert(empty.removeValue("one") == null);
	}

	public void testIdenticalValuesRemoval()
	{
		full.put("five", new Integer(1));
		assert(full.removeValue(new Integer(1)) != null);
		assert(full.values().size() == full.keys().size());
		assert(full.keys().size() == 5);
	}

}
